#include "parse.h"
#include <string.h>
#include <malloc.h>

ParseLine::ParseLine() {
	pHead = NULL;
	pPrev = NULL;
	pNext = NULL;
	ptHead = NULL;
	line_number = 0;
}

ParseToken *ParseLine::Parse() {
	// return a parsed list of tokens, simple form
	return(Parse(NULL,NULL));
}

ParseToken *ParseLine::Parse(char *sep, char *com) {
	char *Sep;
	char *Com;
	int nSep;
	int nCom;
	int nEnd;
	int i;
	int j;
	char c;
	char token_buffer[1024];
	int cmt = 0;
	int token_loc = 0;
	int type;

	ParseToken *t_pHead = NULL;
	ParseToken *t_pPrev = NULL;
	ParseToken *t_pToken;
	
	//if(t_pToken) t_pToken->ReleaseAll();

	if(!line_buffer) return(NULL);

	if(!sep)	
		Sep = "\n\r";
	else Sep = sep;
	
	if(!com)
		Com = "";
	else Com = com;

	nSep = strlen(Sep);
	nCom = strlen(Com);

	/* sigma left off here mucho parser left to go */
	nEnd = strlen(line_buffer);
	for(i=0;i<nEnd;i++) {
		// 
		c = line_buffer[i];
		type = PARSE_NONE;

		for(j=0;j<=nSep;j++) {
			if(Sep[j] == c) {
				type = PARSE_SEP;
				break;
			}
		}
		for(j=0;j<nCom;j++) {
			if(Sep[j] == c) {
				type = PARSE_COM;
				break;
			}
		}

		if(type == PARSE_COM) {
			cmt++;
		}
		if(!cmt) {
			if(type == PARSE_SEP) {
			// it's a seperator, so write out the buffer to
			// a new token
			token_buffer[token_loc] = '\000';
			token_loc = 0;
			t_pToken = new ParseToken(t_pPrev);

			if(!t_pHead)  {
				t_pHead = t_pToken;
				t_pPrev = t_pToken;
			} else
				t_pPrev = t_pToken;
			t_pToken->Set(token_buffer);

					
			} else {
			// it's not a seperator, so tag it on the buffer
			token_buffer[token_loc++] = c;
			}
		}
	} // end I loop

	if(token_loc > 0) { // flush any remaining data to token
		token_buffer[token_loc] = '\000';
		t_pToken = new ParseToken(t_pPrev);
		if(!t_pHead) {
			t_pHead = t_pToken;
			t_pPrev = t_pToken;
		} else
			t_pPrev = t_pToken;
		t_pToken->Set(token_buffer);
	}

	ptHead = t_pHead;
	return(t_pHead);
} // end parse token

ParseLine::ParseLine(ParseLine *pH) {
	// construct
	
	if(!pH)  {
		pHead = NULL;
		pPrev = NULL;
		pNext = NULL;
		line_number = 0;
	} else {
		pHead = pH->pHead;
		pPrev = pH;
		pPrev->pNext = this;
		pNext = NULL;
		line_number = pPrev->GetParseLine() + 1;
	}

	line_buffer = NULL;
	
}

ParseLine::~ParseLine() {
	// destruct
	if(line_buffer) free(line_buffer);

}

ParseLine::VerifyNull(char *data) {
	// verify that a null occurs inside our max line length
	int i;

	if(!data) return 0;
	for(i=0;i<MAX_LINE_LEN;i++) {
		if(data[i] == '\000') return(i);
	}

	return(0);
}

ParseLine *ParseLine::GetNextObj() {
	if(this)
	return(pNext);
}

ParseLine *ParseLine::GetPrevObj() {
	if(this)
	return(pPrev);
}

ParseLine *ParseLine::GetNewObj() {
	ParseLine *pt;
	pt = new ParseLine(this);
	return(pt);
}

int ParseLine::GetParseLine() {
	return(line_number);
}

char *ParseLine::Get() {
	return(line_buffer);
}

int ParseLine::Set(char *data) {
	if(!VerifyNull(data)) return(LINE_ERROR_NONULL);
	if(line_buffer) free(line_buffer);
	line_buffer = strdup(data);
}


ParseToken::ParseToken() {
	token_buffer = NULL;
	pHead = this;
	pNext = NULL;
	pPrev  = NULL;
	token_number = 0;
}

ParseToken::GetTokenNumber() {
	return(token_number);
}

ParseToken::ParseToken(ParseToken *pH) {
	token_buffer = NULL;

	if(!pH) {
		pHead = this;
		pNext = NULL;
		pPrev = NULL;
		token_number = 0;
	} else {
		pHead = pH->pHead;
		pNext = NULL;
		pPrev = pH;
		pPrev->pNext = this;
		token_number = pPrev->GetTokenNumber() + 1;
	}

}

ParseToken::~ParseToken() {
}

ParseToken::AddRef() {
	ref_count++;
}

ParseToken::Release() {
	ref_count--;
	if(ref_count == 0) delete this;
}

ParseToken::ReleaseAll() {
	// call release function on all tokens in this chain
	ParseToken *pWork = pHead;
	ParseToken *pWork2;
	
	while(pWork && pWork->pNext) {
		pWork2 = pWork;
		pWork = pWork->pNext;
		pWork2->Release();
	}
	// under normal circumstances this call will clear the
	// entire array. Unnormal circumstances include other functions
	// still having a Ref() to this object
}

ParseToken::Set(char *data) {
	// set a value for this token
	if(token_buffer) free(token_buffer);
	token_buffer = strdup(data);
}

char *ParseToken::Get() {
	// get a value for this token
	if(this)
	return(token_buffer);
}

ParseToken *ParseToken::GetNextObj() {
	if(this)
	return(pNext);
}

ParseToken *ParseToken::GetPrevObj() {
	if(this)
	return(pPrev);
}

ParseToken *ParseToken::GetNewObj() {
	// create a new token bound to us
	ParseToken *newtoken;
	newtoken = new ParseToken(this);
	return(newtoken);
}

ParseToken *ParseToken::GetTokenNumber(int Num) {
	// get a specific token number, assuming it's in index
	// return null if it's not possible

	ParseToken *pRet = NULL;

	if(Num > token_number) pRet = this;
	else pRet = pHead;

	while(pRet->pNext && pRet->token_number != Num) 
		pRet = pRet->pNext;

	if(pRet->token_number == Num) 
		return(pRet); // purrrr...
	
	return(NULL); // hoowowwwwwllL!
}

int ParseToken::GetMaxTokenNumber() {
	ParseToken *pWork = this;
	while(pWork->pNext) pWork = pWork->pNext;
	return(pWork->token_number);
}

